from .config import MetaConfig, InvalidConfigException


def merge(*configs):
    """Merge several dicts to produce one.

    If a key is present in two or more dicts, the value of the lattest
    occurence is took.
    """
    d = {}
    for conf in configs:
        d.update(conf)
    return d


class ManyConfig(MetaConfig):
    """Pull configuration from many others.

    This class is the real plus-value of ManyConfig. It takes some
    configuration sources, load them and merge them in a single configuration.

    Configurations are loaded in the given order, and new values for the same
    configuration key overrides older ones.

    This allow implementation of Bash-like configurations, where multiple files
    are read and each one take precedence over the last one.

    :param metaconfigs: A list of configurations to pull values from
    """

    def __init__(self, *metaconfigs, **kwargs):
        self.metaconfigs = metaconfigs
        super(ManyConfig, self).__init__(**kwargs)

    def _load(self):
        return merge(*[mc.load() for mc in self.metaconfigs])


class AnyConfig(ManyConfig):
    """Pull configuration from the first existing source

    It will iter through the different MetaConfig given and yield the first
    non-empty valid configuration.

    :param metaconfigs: A list of configurations to pull values from
    """

    def _load(self):
        for metaconfig in self.metaconfigs:
            config = metaconfig.load()
            if config:
                return config



import contextlib
import os
import sys

import pytest

from marshmallow import Schema as _Schema
from marshmallow.fields import String, Integer

sys.path.insert(0, os.curdir)


@pytest.fixture(scope="session")
def Schema():
    class schema(_Schema):
        foo = String(required=True)
        bar = Integer()

    return schema


@contextlib.contextmanager
def tmp_env(**env):
    old = {key: os.environ[key] for key in env.keys() if key in os.environ}
    to_remove = [key for key in env.keys() if key not in os.environ]
    os.environ.update(env)
    yield os.environ
    os.environ.update(old)
    for key in to_remove:
        os.environ.pop(key)



class InvalidConfigException(Exception):
    """Exception raises when a configuration is invalid."""

    def __init__(self, message, errors):
        super(InvalidConfigException, self).__init__(message)
        self.errors = errors


class MetaConfig(object):
    """Base class for metaconfigurations.

    A schema passed at instantiation override any schema set at level class.

    :param bool silent: Don't raise exceptions on invalid configurations
    :param schema: A marshmallow schema to validate loaded configuration
    """

    #: A marshmallow schema to validate loaded configuration
    schema = None

    def __init__(self, silent=False, schema=None):
        self.silent = silent
        if schema:
            self.schema = schema

    def load(self):
        """Load the configuration

        :return dict: Dict representing the configuration
        """
        data = self._load()
        if self.schema:
            data, errors = self.schema.load(data)
            if errors and self.silent:
                return {}
            elif errors:
                raise InvalidConfigException("Invalid configuration", errors)
        return data

    def _load(self):  # pragma: no cover
        """Method to override to implement actual loading behaviour"""
        pass



import pytest

from marshmallow import Schema

from manyconfig import MetaConfig, InvalidConfigException


@pytest.fixture
def config_dict():
    return {"foo": "wow", "bar": "42"}


@pytest.fixture()
def TestMetaConfig(Schema, config_dict):

    class TestMetaConfig(MetaConfig):
        schema = Schema()

        def _load(self):
            return config_dict

    return TestMetaConfig


def test_schema_loading(TestMetaConfig):
    """Test that schema.load is used

    See the bar string become an integer
    """
    metaconfig = TestMetaConfig()
    config = metaconfig._load()
    assert not isinstance(config["bar"], int)
    config = metaconfig.load()
    assert isinstance(config["bar"], int)


def test_schema_overide(TestMetaConfig):
    class OtherSchema(Schema):
        pass

    schema = OtherSchema()
    metaconfig = TestMetaConfig(schema=schema)
    assert metaconfig.schema is schema


@pytest.mark.parametrize("config_dict", [{"foo": 4}])
def test_schema_loading_error(TestMetaConfig):
    """Test that schema.load raise an error if the config is not valid"""
    metaconfig = TestMetaConfig()
    with pytest.raises(InvalidConfigException):
        metaconfig.load()


@pytest.mark.parametrize("config_dict", [{"foo": 4}])
def test_schema_loading_silent_error(TestMetaConfig):
    """Test that schema.load raise an error if the config is not valid"""
    metaconfig = TestMetaConfig(silent=True)
    config = metaconfig.load()
    assert config == {}



import json

import pytest

from manyconfig import FileMetaConfig
from manyconfig.file import InvalidFormatError


def test_invalid_format():
    with pytest.raises(InvalidFormatError):
        FileMetaConfig("foo", str())


def test_json(tmpdir):
    d = {"foo": 42, "bar": "Olive&Tom"}
    f = tmpdir.join("foo.json")
    s = json.dumps(d)
    f.write(s)
    metaconfig = FileMetaConfig("json", str(f))
    assert metaconfig.load() == d


ini_text = """[foo]
bar = baz
foo = bar
baz = foo
"""


def test_ini(tmpdir):
    f = tmpdir.join("foo.ini")
    f.write(ini_text)
    metaconfig = FileMetaConfig("ini", str(f))
    assert metaconfig.load() == {"foo": {"bar": "baz", "foo": "bar",
                                         "baz": "foo"}}



from manyconfig import EnvironmentMetaConfig

from tests.conftest import tmp_env


class test_namespace():
    metaconfig = EnvironmentMetaConfig("FOO_")
    with tmp_env(**{"FOO_FOO": "1", "FOO_BAR": "2", "FOO_BAZ": "3"}):
        config = metaconfig.load()
    assert config == {"foo": "1", "bar": "2", "baz": "3"}



from manyconfig import EnvironmentMetaConfig
from manyconfig.manyconfig import merge, ManyConfig, AnyConfig

from tests.conftest import tmp_env


def test_merge():
    d1 = {"foo": 1, "bar": 1}
    d2 = {"foo": 2}
    assert merge(d1, d2) == {"foo": 2, "bar": 1}


def test_ManyConfig():
    env1 = EnvironmentMetaConfig("FOO_")
    env2 = EnvironmentMetaConfig("BAR_")
    metaconfig = ManyConfig(env1, env2)
    with tmp_env(FOO_FOO="1", FOO_BAR="1", BAR_FOO="2", BAR_BAZ="2"):
        assert metaconfig.load() == {"foo": "2", "bar": "1", "baz": "2"}


def test_AnyConfig():
    env1 = EnvironmentMetaConfig("FOO_")
    env2 = EnvironmentMetaConfig("BAR_")
    env3 = EnvironmentMetaConfig("BAZ_")
    metaconfig = AnyConfig(env1, env2, env3)
    with tmp_env(BAR_FOO="2", BAZ_BAR="3"):
        assert metaconfig.load() == {"foo": "2"}



import json
try:
    from configparser import ConfigParser
except ImportError:  # pragma: no cover
    from ConfigParser import ConfigParser

from .config import MetaConfig


class InvalidFormatError(ValueError):
    pass


class DecoratorDict(dict):
    def add(self, key):
        def decorator(f):
            self[key] = f
            return f
        return decorator


format_parsers = DecoratorDict({"json": json.load})


class FileMetaConfig(MetaConfig):
    """Pull configuration from a file.

    :param filepath: the path of the configuration file.
    :param bool binary: Open the file in binary mode.
    """

    def __init__(self, format, filepath, binary=False, **kwargs):
        if format not in format_parsers.keys():
            raise InvalidFormatError("format not supported")
        self.format = format
        self.filepath = filepath
        self.mode = 'rb' if binary else 'r'
        super(FileMetaConfig, self).__init__(**kwargs)

    def _load(self):
        """Open the given file and call the adapted parser."""
        parser = format_parsers.get(self.format)
        with open(self.filepath, mode=self.mode) as file_object:
            config = parser(file_object)
        return config


@format_parsers.add("ini")
def parse_ini(file_object):
    """Parse the INI in the file."""
    config_parser = ConfigParser()
    config_parser.readfp(file_object)
    return {section: dict(config_parser.items(section))
            for section in config_parser.sections()}



"""
Manyconfig
"""

import os
from setuptools import find_packages, setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="manyconfig",
    version="0.1.1",
    author="Shir0kamii",
    author_email="shir0kamii@gmail.com",
    description="An easy-to-use declarative loader of configuration",
    long_description=read("README.rst"),
    url="https://github.com/Shir0kamii/manyconfig",
    download_url="https://github.com/Shir0kamii/manyconfig/tags",
    platforms="any",
    packages=find_packages(),
    classifiers=[
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ]
)



import os

from .config import MetaConfig


class EnvironmentMetaConfig(MetaConfig):
    """Pull configuration from environment

    An environment variable is considered to be in a namespace if it begins by
    it. That is, FOO_BAR is part of the FOO_ namespace.

    All environment variables of the given namespace will be collected, and the
    namespace removed from its beginning. It is then inserted in the
    configuration with its value.

    :param namespace: The namespace to pull from
    """

    def __init__(self, namespace, **kwargs):
        self.namespace = namespace
        super(EnvironmentMetaConfig, self).__init__(**kwargs)

    def _load(self):
        ns_len = len(self.namespace)
        return {key[ns_len:].lower(): value
                for key, value in os.environ.items()
                if key.startswith(self.namespace)}
